package name.nathanmcrae.numbersstation;

import java.util.Optional;
import javafx.collections.ListChangeListener;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.ListCell;
import javafx.scene.control.ListView;
import javafx.scene.control.TextInputDialog;
import javafx.event.ActionEvent;
import javafx.event.Event;
import javafx.scene.Node;
import javafx.stage.Stage;

public class StationSelectionController {

    private MainSettings settings;

    @FXML
    private ListView<StationSettings> stationListView;

    private ObservableList<StationSettings> stationList;

    public StationSelectionController() {
        stationList = FXCollections.observableArrayList();
        stationList.addListener((ListChangeListener<StationSettings>) change -> onStationListChanged(change));
    }

    @FXML
    private void initialize() {
        // Set custom cell factory to display station name
        stationListView.setCellFactory(param -> new ListCell<StationSettings>() {
            @Override
            protected void updateItem(StationSettings item, boolean empty) {
                super.updateItem(item, empty);
                if (empty || item == null || item.getName() == null) {
                    setText(null);
                } else {
                    setText(item.getName());
                }
            }
        });
    }

    public void setSettings(MainSettings newSettings) {
        settings = newSettings;
        stationList.addAll(settings.getStations());
        stationListView.setItems(stationList);
    }

    @FXML
    private void handleAddButtonPress(ActionEvent event) {
        TextInputDialog dialog = new TextInputDialog();
        dialog.setTitle("New Station");
        dialog.setHeaderText("Add a New Station");
        dialog.setContentText("Please enter the station name:");

        Optional<String> result = dialog.showAndWait();
        result.ifPresent(stationName -> {
            StationSettings newStation = new StationSettings(stationName);
            stationList.add(newStation);
        });
    }

    @FXML
    private void handleRemoveButtonPress(ActionEvent event) {
        int selectedIndex = stationListView.getSelectionModel().getSelectedIndex();
        if (selectedIndex >= 0) {
            stationList.remove(selectedIndex);
        }
    }

    @FXML
    private void handleSelectButtonPress(Event e) {
        Node node = (Node) e.getSource();
        Stage stage = (Stage) node.getScene().getWindow();
        StationSettings selectedStation = stationListView.getSelectionModel().getSelectedItem();

        if (selectedStation != null) {
            stage.setUserData(selectedStation.getName());
            System.out.println("Selected Station: " + selectedStation.getName());
        }
        stage.close();
    }

    private void onStationListChanged(ListChangeListener.Change<? extends StationSettings> change) {
        while (change.next()) {
            if (change.wasAdded()) {
                for (StationSettings addedStation : change.getAddedSubList()) {
                    System.out.println("Added: " + addedStation.getName());
                }
            }
            if (change.wasRemoved()) {
                for (StationSettings removedStation : change.getRemoved()) {
                    System.out.println("Removed: " + removedStation.getName());
                }
            }
        }
        settings.getStations().clear();
        settings.getStations().addAll(stationList);
        settings.save();
    }
}
